import requests
import logging

class Component(object):
    def __init__(self,name,rieapie,parent=None):
        self.name=name
        self.parent=parent
        self.rieapie=rieapie

    def __getattribute__(self,key):
        try:
            return object.__getattribute__(self,key)
        except:
            return Component(key,self.rieapie,self)

    def full_path(self):
        return "/".join([self.rieapie.base_url,self.path()])

    def path(self):
        path = []
        cur = self
        while cur.parent:
            path+=[cur.name]
            cur = cur.parent
        path += [cur.name]
        return "/".join(reversed(path))

    def __repr__(self):
        return self.path().replace("/",".")

    def __call__(self,ext=""):
        if ext:
            self.name+=".%s"%ext
        return self

    def __getitem__(self,key):
        return Component(key,self.rieapie,self)

    def get(self,**kwargs):
        url,params,_=self.rieapie.pre_request(self.full_path(),kwargs,None)
        resp=requests.get(url,params=params,headers=self.rieapie.headers)
        return resp.json()

    def delete(self, **kwargs):
        url,params,_=self.rieapie.pre_request(self.full_path(),kwargs,None)
        resp=requests.delete(url,params=params,headers=self.rieapie.headers)
        return resp.json()

    def create(self, **kwargs):
        url,params,data = self.rieapie.pre_request(self.full_path(),{},kwargs)
        resp=requests.put(url,params=params,data=data,headers=self.rieapie.headers)
        return resp.json()

    def update(self, **kwargs):
        url,params,data = self.rieapie.pre_request(self.full_path(),{},kwargs)
        resp=requests.post(url,params=params,data=data,headers=self.rieapie.headers)
        return resp.json()


class Api(object):
    def __init__(self,base_url,request_headers={}, debug=False):
        self.base_url=base_url
        self.headers=request_headers
        if debug:
            logging.basicConfig(level=logging.DEBUG)

    def __getattribute__(self,key):
        try:
            return object.__getattribute__(self,key)
        except:
            return Component(key,self,None)

    def pre_request(self,url,params,data):
        return url,params,data




import rieapie
import urllib
import base64
import requests
class Twitter(rieapie.Api):
    def __init__(self, consumer_key, consumer_secret):
        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.base_url = "https://api.twitter.com/1.1"
        bearer_token = self.obtain_bearer_token()
        headers = {"Authorization":"Bearer %s" % bearer_token}
        rieapie.Api.__init__(self
                                , self.base_url
                                , request_headers = headers)
    def obtain_bearer_token(self):
        """
        request for an access token from twitter
        """
        _e_key = urllib.quote(self.consumer_key)
        _e_secret = urllib.quote(self.consumer_secret)
        credentials = base64.b64encode("%s:%s" % ( _e_key, _e_secret ))
        resp = requests.post("https://api.twitter.com/oauth2/token"
                , params = {"grant_type":"client_credentials"}
                , headers = {"Authorization": "Basic %s" % credentials} )
        resp_json = resp.json()
        if not resp_json.has_key("access_token"):
            raise UserWarning("failed to obtain access token : %s" % resp_json["errors"])
        return resp.json()["access_token"]




"""
setup.py for rieapie

"""
__author__ = "Ali-Akber Saifee"
__email__ = "ali@indydevs.org"
__copyright__ = "Copyright 2013, Ali-Akber Saifee"

from setuptools import setup, find_packages
import os
import sys
import rieapie
this_dir = os.path.abspath(os.path.dirname(__file__))
REQUIREMENTS = filter(None, open(os.path.join(this_dir, 'requirements.txt')).read().splitlines())
extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True


setup(
    name='rieapie',
    author = __author__,
    author_email = __email__,
    license = open('LICENSE.txt').read(),
    url="http://github.com/alisaifee/rieapie",
    zip_safe = False,
    version=rieapie.version,
    classifiers=[k for k in open('CLASSIFIERS').read().split('\n') if k],
    include_package_data = True,
    install_requires = REQUIREMENTS,
    description='Easy REST api wrapper',
    long_description=open('README.rst').read(),
    packages = find_packages(),
    **extra
)

